@@ -19,6 +19,9 @@ class Haplotype
         Haplotype(const std::string& ref_name,
                   const size_t ref_position,
                   const std::string& ref_sequence);
+
+        //
+        ~Haplotype();
         
         // get the sequence of the haplotype
         const std::string& get_sequence() const { return m_sequence; } 
@@ -39,6 +42,10 @@ class Haplotype
         // add a variant into the haplotype
         // returns true if the variant is successfully added to the haplotype
         bool apply_variant(const Variant& v);
+        
+        // add multiple variants into the haplotype
+        // returns true if all variants are successfully added to the haplotype
+        bool apply_variants(const std::vector<Variant>& variants);
 
         // return all the variants on this haplotype
         std::vector<Variant> get_variants() const { return m_variants; }
@@ -66,6 +73,8 @@ class Haplotype
         // This mimics std::lower_bound
         size_t _find_derived_index_by_ref_lower_bound(size_t ref_index) const;
 
+        void print_debug_info() const;
+
         //
         // data
         //
